package metadata
{
    import flash.events.Event;
    
    import org.spicefactory.parsley.config.ObjectDefinitionDecorator;
    import org.spicefactory.parsley.core.scope.ScopeName;
    import org.spicefactory.parsley.dsl.ObjectDefinitionBuilder;
    
    [Metadata(types="method,property", multiple="true")]
    /**
     *  The Bindable class is a Parsley metadata processor that piggy-backs off
     *  of the defined Bindable metadata in Flex.  It adds a bindingDependency
     *  property so that you can specify a property that this particular getter 
     *  is dependent on.  For instance: 
     *  [BindableDependency(event="property1Changed", dependentProperty="myModel.property1")].
     */
    public class Bindable implements ObjectDefinitionDecorator 
    {
        //--------------------------------------------------------------------------
        //
        //  Properties
        //
        //--------------------------------------------------------------------------
        
        //----------------------------------
        //  event
        //----------------------------------
        
        [DefaultProperty]
        /**
         *  The event name to use for this bindable property.
         * 
         *  <p>When the bindingDependency property changes, an event 
         *  with this name will be dispatched.  When a consumer binds 
         *  to this property, they will be listening for events 
         *  of this name.</p>
         */
        public var event:String;
        
        [Target]
        /**
         *  The getter or method we are targetting.
         */
        public var property:String;
        
        /**
         *  The property this binding is dependent on.  This should be specified 
         *  in dot-notation.  For instance, dependentProperty="myModel.property1".
         */
        public var dependentProperty:String;
        
        
        //--------------------------------------------------------------------------
        //
        //  Methods
        //
        //--------------------------------------------------------------------------
        
        /**
         *  Method called to process the Metadata.
         * 
         *  @param builder Object definition builder
         */
        public function decorate (builder:ObjectDefinitionBuilder) : void 
        {
            // if bindingDependency is defined, we'll set up code to run 
            // on each instance later on.  However, if no binding property, don't 
            // worry about anything
            if (dependentProperty)
            {
                builder.lifecycle().processorFactory(
                    BindingDependencyProcessor.newFactory(property, dependentProperty, event));
            }
        }
    }
}